// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/driver/ioctl.c $
// $Rev: 54401 $
// $Date: 2024-05-02 09:37:04 -0500 (Thu, 02 May 2024) $

// HPDI32-COS: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	INIT_START				D0	// BCR



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:							arg[0]	= HPDI32COS_IOCTL_QUERY_ERROR;	break;
		case HPDI32COS_QUERY_CLOCK_DIV_MAX:	arg[0]	= dev->cache.clock_div_max;		break;
		case HPDI32COS_QUERY_COUNT:			arg[0]	= HPDI32COS_IOCTL_QUERY_LAST;	break;
		case HPDI32COS_QUERY_DEVICE_TYPE:	arg[0]	= dev->board_type;				break;
		case HPDI32COS_QUERY_FIFO_SIZE_RX:	arg[0]	= dev->cache.fifo_size_rx;		break;
		case HPDI32COS_QUERY_FORM_FACTOR:	arg[0]	= dev->cache.form_factor;		break;
		case HPDI32COS_QUERY_FREF_DEFAULT:	arg[0]	= dev->cache.fref_default;		break;
		case HPDI32COS_QUERY_FREF_MAX:		arg[0]	= dev->cache.fref_max;			break;
		case HPDI32COS_QUERY_JUMPER_ON:		arg[0]	= dev->cache.jumper_on;			break;
		case HPDI32COS_QUERY_JUMPER_QTY:	arg[0]	= dev->cache.jumper_qty;		break;
		case HPDI32COS_QUERY_JUMPER_VAL:	arg[0]	= dev->cache.jumper_val;		break;
		case HPDI32COS_QUERY_MODEL:			arg[0]	= dev->cache.model;				break;
	}

	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	int		i;
	long	ms		= 2;
	u32		reg;
	int		ret		= 0;
	int		tmp;
	VADDR_T	va		= dev->vaddr.gsc_bcr_32;

	// The HPDI32COS doesn't have an Initialization Done interrupt so we don't
	// involve interrupts here.

	// Initiate initialization.
	os_reg_mem_tx_u32(dev, va, INIT_START);

	// Manually wait for completion.
	tmp	= gsc_poll_u32(dev, ms, va, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion.
	reg	= os_reg_mem_rx_u32(dev, va);

	if (reg & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _byte_enable(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_bcr_32, 15, 12);
	return(ret);
}



//*****************************************************************************
static int _cc_d0_d6_state(dev_data_t* dev, s32* arg)
{
	s32	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0x7F, dev->vaddr.gsc_bsr_32, 6, 0);
	return(ret);
}



//*****************************************************************************
static int _cc_d5_d6(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_CC_D5_D6_RW,
		HPDI32COS_CC_D5_D6_RO
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _clock_divider(dev_data_t* dev, s32* arg)
{
	s32	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_scdr_32, 15, 0);
	return(ret);
}



//*****************************************************************************
static int _cos_detected(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_COS_DETECTED_NO,
		HPDI32COS_COS_DETECTED_YES
		-1	// terminate list
	};

	int	ret;

	if (arg[0] == HPDI32COS_COS_DETECTED_CLEAR)
	{
		// In early firmware versions (0x02 and prior) the COS Detected status
		// is cleared with any write to the Interrupt Status Register. Thus, if
		// any interrupt is enabled and triggered, then this service will
		// return the NO option.
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_isr_32, 0);
		arg[0]	= -1;
	}

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 26, 26);
	return(ret);
}



//*****************************************************************************
static int _counter_zero(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_COUNTER_ZERO_NO,
		HPDI32COS_COUNTER_ZERO_YES
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 25, 25);
	return(ret);
}



//*****************************************************************************
static int _event_count(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ecr_32);
	return(0);
}



//*****************************************************************************
static int _evnt_cnt_ini_get(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ecivr_32);
	return(0);
}



//*****************************************************************************
static int _evnt_cnt_ini_set(dev_data_t* dev, u32* arg)
{
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_ecivr_32, arg[0]);
	return(0);
}



//*****************************************************************************
static int _fifo_word_count(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rfwcr_32);
	return(0);
}



//*****************************************************************************
static int _irq_config_edge(dev_data_t* dev, s32* arg)
{
	u32	ielr;
	int	ret		= 0;

	if (arg[0] == -1)
	{
		ielr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ielr_32);
		arg[0]	= ielr & IRQ_MASK;
	}
	else if (arg[0] & ~IRQ_MASK)
	{
		ret	= -EINVAL;
	}
	else
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ielr_32, arg[0], IRQ_MASK);
	}

	return(ret);
}



//*****************************************************************************
static int _irq_config_high(dev_data_t* dev, s32* arg)
{
	u32	ihlr;
	int	ret		= 0;

	if (arg[0] == -1)
	{
		ihlr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_ihlr_32);
		arg[0]	= ihlr & IRQ_MASK;
	}
	else if (arg[0] & ~IRQ_MASK)
	{
		ret	= -EINVAL;
	}
	else
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ihlr_32, arg[0], IRQ_MASK);
	}

	return(ret);
}



//*****************************************************************************
static int _irq_enable(dev_data_t* dev, s32* arg)
{
	u32	icr;
	int	ret		= 0;

	if (arg[0] == -1)
	{
		icr		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_icr_32);
		arg[0]	= icr & IRQ_MASK;
	}
	else if (arg[0] & ~IRQ_MASK)
	{
		ret	= -EINVAL;
	}
	else
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, arg[0], IRQ_MASK);
	}

	return(ret);
}



//*****************************************************************************
static int _la_trig_word_get(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_latr_32);
	return(0);
}



//*****************************************************************************
static int _la_trig_word_set(dev_data_t* dev, u32* arg)
{
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_latr_32, arg[0]);
	return(0);
}



//*****************************************************************************
static int _la_triggered(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_LA_TRIGGERED_NO,
		HPDI32COS_LA_TRIGGERED_YES
		-1	// terminate list
	};

	int	ret;

	// The LA Triggered status is cleared by setting Start Rx to 0.
	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 23, 23);
	return(ret);
}



//*****************************************************************************
static int _loopback(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_LOOPBACK_DISABLE,
		HPDI32COS_LOOPBACK_ENABLE
		-1	// terminate list
	};

	int	ret;

	// Rx Start is inhibited if Tx Enable is set and Loopback is disabled.
	// It doesn't affect the rx Statrt bit, but it does prevent Rx Start from
	// taking place.

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 7, 7);
	return(ret);
}



//*****************************************************************************
static int _mode(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_MODE_COS,
		HPDI32COS_MODE_LA
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _rx_data(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rdir_32);
	return(0);
}



//*****************************************************************************
static int _rx_data_mask_get(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_dmr_32);
	return(0);
}



//*****************************************************************************
static int _rx_data_mask_set(dev_data_t* dev, u32* arg)
{
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_dmr_32, arg[0]);
	return(0);
}



//*****************************************************************************
static int _rx_fifo_ae(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_rar_32, 15, 0);

	// Reset the FIFO so the setting will take affect.

	if (ret == 0)
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D2, D2);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, D2, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _rx_fifo_af(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_rar_32, 31,16);

	// Reset the FIFO so the setting will take affect.

	if (ret == 0)
	{
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D2, D2);
		os_time_us_delay(10);
		gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, D2, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _rx_fifo_over(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_FIFO_ERROR_NO,
		HPDI32COS_FIFO_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	if (arg[0] == HPDI32COS_FIFO_ERROR_CLEAR)
	{
		// The Overflow status is also cleared by a Board Reset or an Rx FIFO Reset.

		if ((dev->cache.gsc_frr_32 & 0xFF) <= 0x02)
		{
			// In these versions there is firmware bug.
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bsr_32, D23);
		}
		else
		{
			// The bug is fixed in these versions.
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bsr_32, D20);
		}

		arg[0]	= -1;
	}

	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 20, 20);
	return(ret);
}



//*****************************************************************************
static int _rx_fifo_reset(dev_data_t* dev, void* arg)
{
	// The Rx FIFO is a physical FIFO chip and may take longer to reset.
	// This also clears the Overflow and Underflow status bits.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D2, D2);
	os_time_us_delay(10);
	gsc_poll_u32(dev, 10, dev->vaddr.gsc_bcr_32, D2, 0);
	return(0);
}



//*****************************************************************************
static int _rx_fifo_status(dev_data_t* dev, s32* arg)
{
	u32	bsr;

	bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

	if ((bsr & D15) == 0)
		arg[0]	= HPDI32COS_FIFO_STATUS_FULL;
	else if ((bsr & D14) == 0)
		arg[0]	= HPDI32COS_FIFO_STATUS_AF;
	else if ((bsr & D12) == 0)
		arg[0]	= HPDI32COS_FIFO_STATUS_EMPTY;
	else if ((bsr & D13) == 0)
		arg[0]	= HPDI32COS_FIFO_STATUS_AE;
	else
		arg[0]	= HPDI32COS_FIFO_STATUS_MEDIUM;

	return(0);
}



//*****************************************************************************
static int _rx_fifo_under(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_FIFO_ERROR_NO,
		HPDI32COS_FIFO_ERROR_YES,
		-1	// terminate list
	};

	int	ret;

	if (arg[0] == HPDI32COS_FIFO_ERROR_CLEAR)
	{
		// The Underflow status is cleared by a Board Reset or an Rx FIFO Reset.

		if ((dev->cache.gsc_frr_32 & 0xFF) <= 0x02)
		{
			// In these versions there is firmware bug.
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bsr_32, D22);
		}
		else
		{
			// The bug is fixed in these versions.
			os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bsr_32, D19);
		}

		arg[0]	= -1;
	}

	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bsr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



//*****************************************************************************
static int _rx_io_bmdma_trsh(dev_data_t* dev, void* arg)
{
	int	ret;

	ret		= gsc_s32_range_var(arg, 0, dev->cache.fifo_size_rx, &dev->io.rx.bmdma_threshold);
	return(ret);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_over(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_IO_ERROR_IGNORE,
		HPDI32COS_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.over_check);
	return(ret);
}



//*****************************************************************************
static int _rx_io_pio_trsh(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret		= gsc_s32_range_var(arg, 0, S32_MAX, &dev->io.rx.pio_threshold);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			HPDI32COS_IO_TIMEOUT_MIN,
			HPDI32COS_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_under(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_IO_ERROR_IGNORE,
		HPDI32COS_IO_ERROR_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.under_check);
	return(ret);
}



//*****************************************************************************
static int _rx_start(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_RX_START_NO,
		HPDI32COS_RX_START_YES
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _rx_state(dev_data_t* dev, s32* arg)
{
	u32	bsr;

	bsr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bsr_32);

	// Rx Start is inhibited if Tx Enable is set and Loopback is disabled.
	// It doesn't affect the rx Statrt bit, but it does prevent Rx Start from
	// taking place.

	if ((bsr & D4) && ((bsr & D7) == 0))
		arg[0]	= HPDI32COS_RX_STATE_DISABLED;
	else if (bsr & D24)
		arg[0]	= HPDI32COS_RX_STATE_RUNNING;
	else if (bsr & D22)
		arg[0]	= HPDI32COS_RX_STATE_STOPPED;
	else
		arg[0]	= HPDI32COS_RX_STATE_DISABLED;

	return(0);
}



//*****************************************************************************
static int _rx_word_count(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rwcr_32);
	return(0);
}



//*****************************************************************************
static int _tx_data_get(dev_data_t* dev, u32* arg)
{
	arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_tdor_32);
	return(0);
}



//*****************************************************************************
static int _tx_data_set(dev_data_t* dev, u32* arg)
{
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_tdor_32, arg[0]);
	return(0);
}



//*****************************************************************************
static int _tx_enable(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		HPDI32COS_TX_ENABLE_NO,
		HPDI32COS_TX_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	// Rx Start is inhibited if Tx Enable is set and Loopback is disabled.
	// It doesn't affect the rx Statrt bit, but it does prevent Rx Start from
	// taking place.

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 4, 4);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ HPDI32COS_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ HPDI32COS_IOCTL_REG_WRITE,		(void*) gsc_reg_write_ioctl		},
	{ HPDI32COS_IOCTL_REG_MOD,			(void*) gsc_reg_mod_ioctl		},
	{ HPDI32COS_IOCTL_QUERY,			(void*) _query					},
	{ HPDI32COS_IOCTL_INITIALIZE,		(void*) initialize_ioctl		},
	{ HPDI32COS_IOCTL_BYTE_ENABLE,		(void*) _byte_enable			},
	{ HPDI32COS_IOCTL_CC_D0_D6_STATE,	(void*) _cc_d0_d6_state			},
	{ HPDI32COS_IOCTL_CC_D5_D6,			(void*) _cc_d5_d6				},
	{ HPDI32COS_IOCTL_CLOCK_DIVIDER,	(void*) _clock_divider			},
	{ HPDI32COS_IOCTL_COS_DETECTED,		(void*) _cos_detected			},
	{ HPDI32COS_IOCTL_COUNTER_ZERO,		(void*) _counter_zero			},
	{ HPDI32COS_IOCTL_EVENT_COUNT,		(void*) _event_count			},
	{ HPDI32COS_IOCTL_EVNT_CNT_INI_GET,	(void*) _evnt_cnt_ini_get		},
	{ HPDI32COS_IOCTL_EVNT_CNT_INI_SET,	(void*) _evnt_cnt_ini_set		},
	{ HPDI32COS_IOCTL_FIFO_WORD_COUNT,	(void*) _fifo_word_count		},
	{ HPDI32COS_IOCTL_IRQ_CONFIG_EDGE,	(void*) _irq_config_edge		},
	{ HPDI32COS_IOCTL_IRQ_CONFIG_HIGH,	(void*) _irq_config_high		},
	{ HPDI32COS_IOCTL_IRQ_ENABLE,		(void*) _irq_enable				},
	{ HPDI32COS_IOCTL_LA_TRIG_WORD_GET,	(void*) _la_trig_word_get		},
	{ HPDI32COS_IOCTL_LA_TRIG_WORD_SET,	(void*) _la_trig_word_set		},
	{ HPDI32COS_IOCTL_LA_TRIGGERED,		(void*) _la_triggered			},
	{ HPDI32COS_IOCTL_LOOPBACK,			(void*) _loopback				},
	{ HPDI32COS_IOCTL_MODE,				(void*) _mode					},
	{ HPDI32COS_IOCTL_RX_DATA,			(void*) _rx_data				},
	{ HPDI32COS_IOCTL_RX_DATA_MASK_GET,	(void*) _rx_data_mask_get		},
	{ HPDI32COS_IOCTL_RX_DATA_MASK_SET,	(void*) _rx_data_mask_set		},
	{ HPDI32COS_IOCTL_RX_FIFO_AE,		(void*) _rx_fifo_ae				},
	{ HPDI32COS_IOCTL_RX_FIFO_AF,		(void*) _rx_fifo_af				},
	{ HPDI32COS_IOCTL_RX_FIFO_OVER,		(void*) _rx_fifo_over			},
	{ HPDI32COS_IOCTL_RX_FIFO_RESET,	(void*) _rx_fifo_reset			},
	{ HPDI32COS_IOCTL_RX_FIFO_STATUS,	(void*) _rx_fifo_status			},
	{ HPDI32COS_IOCTL_RX_FIFO_UNDER,	(void*) _rx_fifo_under			},
	{ HPDI32COS_IOCTL_RX_IO_ABORT,		(void*) _rx_io_abort			},
	{ HPDI32COS_IOCTL_RX_IO_BMDMA_TRSH,	(void*) _rx_io_bmdma_trsh		},
	{ HPDI32COS_IOCTL_RX_IO_MODE,		(void*) _rx_io_mode				},
	{ HPDI32COS_IOCTL_RX_IO_OVER,		(void*) _rx_io_over				},
	{ HPDI32COS_IOCTL_RX_IO_PIO_TRSH,	(void*) _rx_io_pio_trsh			},
	{ HPDI32COS_IOCTL_RX_IO_TIMEOUT,	(void*) _rx_io_timeout			},
	{ HPDI32COS_IOCTL_RX_IO_UNDER,		(void*) _rx_io_under			},
	{ HPDI32COS_IOCTL_RX_START,			(void*) _rx_start				},
	{ HPDI32COS_IOCTL_RX_STATE,			(void*) _rx_state				},
	{ HPDI32COS_IOCTL_RX_WORD_COUNT,	(void*) _rx_word_count			},
	{ HPDI32COS_IOCTL_TX_DATA_GET,		(void*) _tx_data_get			},
	{ HPDI32COS_IOCTL_TX_DATA_SET,		(void*) _tx_data_set			},
	{ HPDI32COS_IOCTL_TX_ENABLE,		(void*) _tx_enable				},
	{ HPDI32COS_IOCTL_WAIT_CANCEL,		(void*) gsc_wait_cancel_ioctl	},
	{ HPDI32COS_IOCTL_WAIT_EVENT,		(void*) gsc_wait_event_ioctl	},
	{ HPDI32COS_IOCTL_WAIT_STATUS,		(void*) gsc_wait_status_ioctl	},
	{ -1, NULL }
};


