// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/driver/main.h $
// $Rev: 54346 $
// $Date: 2024-04-30 14:42:58 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "hpdi32cos.h"

#define	DEV_BAR_SHOW						1
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						1
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_PROC_ID_STR
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT

#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"HPDI32COS"	// Upper case form of the below.
#define	DEV_NAME							"hpdi32cos"	// MUST AGREE WITH HPDI32COS_BASE_NAME

#define	DEV_VERSION							"1.0"	// FOR DEVICE SPECIFIC CODE ONLY!
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK

// WAIT services
#define	DEV_WAIT_GSC_ALL					HPDI32COS_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					HPDI32COS_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						HPDI32COS_WAIT_IO_ALL

#define	IRQ_MASK							0x00FBF07F



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					bmdma_threshold;	// Lower DMA transfer size limit.
	s32					data_size_bits;		// Data size in bits.
	s32					io_mode;			// PIO, BMDMA, DMDMA
	s32					over_check;			// Check overflow before Rx.
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					under_check;		// Check under before Tx/Rx.

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC common create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.
	char				proc_id_str[8];	// User Jumper setting, if supported.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// For read operations.
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_frr_32;		// Firmware Revision Register
		VADDR_T			gsc_bcr_32;		// Board Control Register
		VADDR_T			gsc_bsr_32;		// Board Status Register
		VADDR_T			gsc_ecr_32;		// Event Counter Register
		VADDR_T			gsc_rar_32;		// Rx Almost Register
		VADDR_T			gsc_scdr_32;	// Sample Clock Divider Register
		VADDR_T			gsc_rdfr_32;	// Rx Data FIFO Register
		VADDR_T			gsc_tdor_32;	// Tx Data Output Register
		VADDR_T			gsc_rdir_32;	// Rx Data Input Register
		VADDR_T			gsc_latr_32;	// Logic Analyzer Trigger Register
		VADDR_T			gsc_dmr_32;		// Data Mask Register
		VADDR_T			gsc_ecivr_32;	// Event Counter Initial Value Register
		VADDR_T			gsc_rwcr_32;	// Rx Word Count Register
		VADDR_T			gsc_icr_32;		// Interrupt Control Register
		VADDR_T			gsc_isr_32;		// Interrupt Status Register
		VADDR_T			gsc_rfsr_32;	// Rx FIFO Size Register
		VADDR_T			gsc_rfwcr_32;	// Rx FIFO Word COunt Register
		VADDR_T			gsc_ielr_32;	// Interrupt Edge/Level Register
		VADDR_T			gsc_ihlr_32;	// Interrupt High/Low Register

	} vaddr;

	struct
	{
		u32				gsc_frr_32;			// 0x00	Firmware Revision Register

		s32				clock_div_max;

		u32				fifo_size_rx;		// Rx FIFO Size
		s32				form_factor;		// PCI, PMC, ...
		s32				fref_default;
		s32				fref_max;

		s32				init_ms;			// ms needed for board reset.

		s32				jumper_on;			// What value indicates the jumper is present?
		s32				jumper_qty;			// How many are there?
		s32				jumper_val;			// What is the value for the jumpers.w

		s32				model;				// Which COS model are we accessing?

		s32				reg_fwr_update;		// The old fw read zero when full.

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t bytes);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);

int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
