// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/samples/din/perform.c $
// $Rev: 54381 $
// $Date: 2024-05-01 08:37:50 -0500 (Wed, 01 May 2024) $

// HPDI32-COS: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	DASH_BITS		((u64) 0x1010101010)
#define	DIO_BITS		39
#define	NIBBLE_BITS		((u64) 0x1111111110)



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= hpdi32cos_initialize	(args->fd, -1, 1);
	errs	+= hpdi32cos_tx_enable	(args->fd, -1, 1, HPDI32COS_TX_ENABLE_NO, NULL);
	errs	+= hpdi32cos_byte_enable(args->fd, -1, 1, HPDI32COS_BYTE_ENABLE_NONE, NULL);
	errs	+= hpdi32cos_cc_d5_d6	(args->fd, -1, 1, HPDI32COS_CC_D5_D6_RO, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("  Seq   CC       D31-D24   D23-D16   D15-D8    D7-D0                                    \n");
	printf("  No.   654-3210 7654-3210 7654-3210 7654-3210 7654-3210  Time Stamp      Delta         \n");
	printf("  ====  ================================================  ==============  ==============\n");
}



//*****************************************************************************
static int _monitor(const args_t* args)
{
	u64				bit;
	int				count	= 0;
	long long		delta;
	int				errs	= 0;
	int				i;
	s64				last	= -1;
	time_t			now;
	time_t			t_limit;
	os_time_ns_t	t0;
	os_time_ns_t	ts;
	os_time_ns_t	ts_last;
	u32				v1;
	s32				v2;
	s64				value;

	gsc_label("Digital Input");
	printf("\n");
	gsc_label_level_inc();

	os_time_get_ns(&t0);
	os_time_get_ns(&ts);

	_output_header();
	t_limit	= time(NULL) + args->seconds + 1;

	for (; errs == 0;)
	{
		errs	+= hpdi32cos_rx_data(args->fd, -1, 0, &v1);
		errs	+= hpdi32cos_cc_d0_d6_state(args->fd, -1, 0, &v2);
		value	= (((u64) v2) << 32) | v1;

		if ((count <= 0) || (value != last))
		{
			ts_last	= ts;
			os_time_get_ns(&ts);

			if (count <= 0)
			{
				os_time_get_ns(&t0);
				ts		= t0;
				ts_last	= t0;
			}

			count++;
			printf("  %-4d  ", count);

			for (i = DIO_BITS - 1; i >= 0; i--)
			{
				bit	= ((u64) 0x1) << i;
				printf("%s", (value & bit) ? "1" : "0");

				if (bit & DASH_BITS)
					printf("-");
				else if (bit & NIBBLE_BITS)
					printf(" ");
			}

			delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
					- (1000000000LL * t0.tv_sec + t0.tv_nsec);
			printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

			delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
					- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
			printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

			printf("\n");
			fflush(stdout);
			last	= value;
		}

		now	= time(NULL);

		if (now >= t_limit)
			break;

		if (args->ms_wait)
			os_sleep_ms(args->ms_wait);
	}

	printf("\n");
	gsc_label("Status");
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= _config(args);
	errs	+= _monitor(args);

	return(errs);
}


