// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/samples/rxrate/perform.c $
// $Rev: 54379 $
// $Date: 2024-04-30 16:43:14 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M				(1L * 1024 * 1024)



// variables ******************************************************************

static	u8	_buf[_1M];



//*****************************************************************************
static int _save_line(FILE* file, const char* src, int errs)
{
	int	len;
	int	ret;

	if ((errs) || (src == NULL) || (src[0] == 0))
	{
		errs	= 0;
	}
	else
	{
		len	= strlen(src);
		ret	= fwrite(src, 1, len, file);

		if (ret != len)
		{
			errs	= 1;
			printf("FAIL <---  (fwrite(), ret = %d)\n", ret);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_header(const args_t* args, FILE* file)
{
	char	buf[128];
	int		errs	= 0;

	errs	+= _save_line(file, "# rxrate results\n", errs);

	errs	+= _save_line(file, "# Range min:     ", errs);
	gsc_label_long_comma_buf(args->range_min, buf);
	errs	+= _save_line(file, buf, errs);
	errs	+= _save_line(file, "\n", errs);

	errs	+= _save_line(file, "# Range max:     ", errs);
	gsc_label_long_comma_buf(args->range_max, buf);
	errs	+= _save_line(file, buf, errs);
	errs	+= _save_line(file, "\n", errs);

	errs	+= _save_line(file, "# Range inc:     ", errs);
	gsc_label_long_comma_buf(args->range_inc, buf);
	errs	+= _save_line(file, buf, errs);
	errs	+= _save_line(file, "\n", errs);

	errs	+= _save_line(file, "# Rx mb:         ", errs);
	gsc_label_long_comma_buf(args->rx_mb, buf);
	errs	+= _save_line(file, buf, errs);
	errs	+= _save_line(file, "\n", errs);

	errs	+= _save_line(file, "# Rx Seconds:    ", errs);
	gsc_label_long_comma_buf(args->seconds, buf);
	errs	+= _save_line(file, buf, errs);
	errs	+= _save_line(file, "\n", errs);

	errs	+= _save_line(file, "# Clock Divider: ", errs);
	gsc_label_long_comma_buf(args->rx_mb, buf);
	errs	+= _save_line(file, buf, errs);
	errs	+= _save_line(file, "\n", errs);

	errs	+= _save_line(file, "# I/O Mode:      ", errs);

	switch (args->io_mode)
	{
		default:

				sprintf(buf, "UNKNOWN (%d)\n", args->io_mode);
				break;

		case GSC_IO_MODE_BMDMA:	strcpy(buf, "BMDMA\n");	break;
		case GSC_IO_MODE_DMDMA:	strcpy(buf, "DMDMA\n");	break;
		case GSC_IO_MODE_PIO:	strcpy(buf, "PIO\n");	break;
	}

	errs	+= _save_line(file, buf, errs);

	errs	+= _save_line(file, "#\n", errs);
	errs	+= _save_line(file, "# Rx Bytes  Rate (B/S)\n", errs);
	errs	+= _save_line(file, "# ========  ============\n", errs);
	return(errs);
}



//*****************************************************************************
static int _save_results(const args_t* args, size_t size, long double rate)
{
	static int	tests	= -1;

	char		buf[128];
	int			errs		= 0;
	FILE*		file;
	const char*	name		= "data.txt";

	for (;;)	// convenience loop.
	{
		file	= fopen(name, "a+b");

		if (file == NULL)
		{
			errs	= 1;
			printf("FAIL <---  (fopen(), file = NULL)\n");
			break;
		}

		if (tests < args->tests)
		{
			errs	= _save_header(args, file);
			tests	= args->tests;
		}

		sprintf(buf, "  %8d  %12.3Lf\n", (int) size, rate);
		errs	= _save_line(file, buf, errs);
		fclose(file);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args, size_t size)
{
	os_time_us_t	begin		= { 0, 0 };
	char			buffer[256];
	os_time_us_t	end;
	int				errs		= 0;
	long long		limit		= 1000000LL * args->rx_mb;
	os_time_us_t	now			= { 0, 0 };
	long double		rate		= 0;
	int				ret;
	long long		total		= 0;
	long			us			= 0;

	gsc_label_long_comma_buf(size, buffer);
	strcat(buffer, " Bytes per read");
	gsc_label(buffer);

	for (;;)	// A convenience loop.
	{
		errs	= hpdi32cos_rx_fifo_reset(args->fd, -1, 0);

		if (errs)
		{
			printf("FAIL <---  (hpdi32cos_rx_fifo_reset())\n");
			break;
		}

		os_time_get_us(&begin);
		end			= begin;
		end.tv_sec	+= args->seconds;

		for (;;)	// perform repetative reads
		{
			ret	= hpdi32cos_read(args->fd, _buf, size);

			if (ret < 0)
			{
				errs	= 1;
				printf("FAIL <---  (hpdi32cos_read(), ret %d)\n", ret);
				break;
			}

			if (ret != size)
			{
				errs	= 1;
				printf("FAIL <---  (received only ");
				gsc_label_long_comma(ret);
				printf(" Byte%s)\n", (ret == 1) ? "" : "s");
				break;
			}

			total	+= size;
			os_time_get_us(&now);

			if ((size) && (total < limit))
				continue;

			if ((now.tv_sec > end.tv_sec) ||
				((now.tv_sec == end.tv_sec) && (now.tv_usec > end.tv_usec)))
			{
				break;
			}
		}

		break;
	}

	if (errs == 0)
	{
		us		= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec + 1000000L - begin.tv_usec)
				- 1000000L;
		rate	= (long double) total / us * 1000000;

		if (args->save)
			errs	= _save_results(args, size, rate);
	}

	if (errs == 0)
	{
		printf("PASS  (");

		// Volume =========================================
		gsc_label_long_comma(total);
		printf(" B, ");

		// Duration =======================================
		gsc_label_long_comma(us / 1000000);
		printf(".%06d S, ", (int) (us % 1000000));

		// Rate ===========================================
		gsc_label_long_comma(rate + 0.5);
		printf(" B/S)\n");
	}

	if (args->set_dump)
		hpdi32cos_api_listing(args->fd);

	return(errs);
}



//*****************************************************************************
static void _set_maximum(const args_t* args, size_t* max)
{
	if (args->range)
		max[0]	= args->range_max;
	else
		max[0]	= sizeof(_buf);

	if (max[0] > sizeof(_buf))
	{
		max[0]	= sizeof(_buf);
		gsc_label("REDUCING MAXIMUM");
		printf("new value is ");
		gsc_label_long_comma(max[0]);
		printf("\n");
	}
}



//*****************************************************************************
static void _set_minimum(const args_t* args, size_t max, size_t* min)
{
	if (args->range)
		min[0]	= args->range_min;
	else
		min[0]	= sizeof(_buf);

	if ((min[0] > max) || (min[0] > sizeof(_buf)))
	{
		if (min[0] > max)
			min[0]	= max;
		else
			min[0]	= sizeof(_buf);

		gsc_label("REDUCING MINIMUM");
		printf("new value is ");
		gsc_label_long_comma(min[0]);
		printf("\n");
	}
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	size_t	inc		= args->range_inc;
	size_t	max;
	size_t	min;
	size_t	size;

	errs	+= hpdi32cos_config_la			(args->fd, -1, 1);
	errs	+= hpdi32cos_clock_divider		(args->fd, -1, 1, args->clock_div, NULL);
	errs	+= hpdi32cos_rx_io_bmdma_trsh	(args->fd, -1, 1, 0, NULL);
	errs	+= hpdi32cos_rx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= hpdi32cos_rx_io_over			(args->fd, -1, 1, HPDI32COS_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32cos_rx_io_pio_trsh		(args->fd, -1, 1, 0, NULL);
	errs	+= hpdi32cos_rx_io_under		(args->fd, -1, 1, HPDI32COS_IO_ERROR_IGNORE, NULL);
	errs	+= hpdi32cos_rx_data_mask_set	(args->fd, -1, 1, 0);
	errs	+= hpdi32cos_rx_fifo_af			(args->fd, -1, 1, 0xFFFF, NULL);

	_set_maximum(args, &max);
	_set_minimum(args, max, &min);

	gsc_label("Rate Test");
	printf("Minimum: ");
	gsc_label_long_comma(args->seconds);
	printf(" Second%s, ", (args->seconds == 1) ? "" : "s");
	gsc_label_long_comma(args->rx_mb);
	printf(" MB\n");
	gsc_label_level_inc();

	for (size = min; size <= max; size += inc)
		errs	+= _rate_test(args, size);

	gsc_label_level_dec();
	return(errs);
}



