// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/byte_enable.c $
// $Rev: 54350 $
// $Date: 2024-04-30 14:45:35 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32cos_byte_enable
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32COS_IOCTL_BYTE_ENABLE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32cos_byte_enable(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128]	= "";
	const char*	comma		= "";
	int			errs;
	const char*	ptr			= buf;
	s32			ret;

	if (verbose)
		gsc_label_index("Output Byte Enable", index);

	ret		= hpdi32cos_ioctl(fd, HPDI32COS_IOCTL_BYTE_ENABLE, &set);
	errs	= ret ? 1 : 0;

	if (set & ~HPDI32COS_BYTE_ENABLE_ALL)
	{
		errs	= 1;
		sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
	}
	else if (set == 0)
	{
		ptr	= "None";
	}
	else
	{
		if (set & HPDI32COS_BYTE_ENABLE_D31_D24)
		{
			strcat(buf, "D31-D24");
			comma	= ", ";
		}

		if (set & HPDI32COS_BYTE_ENABLE_D23_D16)
		{
			strcat(buf, comma);
			strcat(buf, "D23-D16");
			comma	= ", ";
		}

		if (set & HPDI32COS_BYTE_ENABLE_D15_D8)
		{
			strcat(buf, comma);
			strcat(buf, "D15-D8");
			comma	= ", ";
		}

		if (set & HPDI32COS_BYTE_ENABLE_D7_D0)
		{
			strcat(buf, comma);
			strcat(buf, "D7-D0");
		}
	}


	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


