// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/query.c $
// $Rev: 54350 $
// $Date: 2024-04-30 14:45:35 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32cos_query
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32COS_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32cos_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case HPDI32COS_QUERY_CLOCK_DIV_MAX:	ptr	= "Max Clock Divider";	break;
		case HPDI32COS_QUERY_COUNT:			ptr	= "Query Count";		break;
		case HPDI32COS_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
		case HPDI32COS_QUERY_FIFO_SIZE_RX:	ptr	= "Rx FIFO Size";		break;
		case HPDI32COS_QUERY_FORM_FACTOR:	ptr	= "Form Factor";		break;
		case HPDI32COS_QUERY_FREF_DEFAULT:	ptr	= "Ref Clock Default";	break;
		case HPDI32COS_QUERY_FREF_MAX:		ptr	= "Ref Clock Maximim";	break;
		case HPDI32COS_QUERY_JUMPER_ON:		ptr	= "Jumper On Value";	break;
		case HPDI32COS_QUERY_JUMPER_QTY:	ptr	= "Jumper Qty";			break;
		case HPDI32COS_QUERY_JUMPER_VAL:	ptr	= "Jumper Value";		break;
		case HPDI32COS_QUERY_MODEL:			ptr	= "Model";				break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= hpdi32cos_ioctl(fd, HPDI32COS_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;
	ptr		= buf;

	switch (query)
	{
		default:

			errs++;
			gsc_label_long_comma_buf(set, buf);
			break;

		case HPDI32COS_QUERY_CLOCK_DIV_MAX:
		case HPDI32COS_QUERY_COUNT:
		case HPDI32COS_QUERY_JUMPER_QTY:

			gsc_label_long_comma_buf(set, buf);
			break;

		case HPDI32COS_QUERY_FREF_DEFAULT:
		case HPDI32COS_QUERY_FREF_MAX:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case HPDI32COS_QUERY_JUMPER_ON:

			if (set)
				strcpy(buf, "1 = On, 0 = Off");
			else
				strcpy(buf, "0 = On, 1 = Off");

			break;

		case HPDI32COS_QUERY_JUMPER_VAL:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case HPDI32COS_QUERY_FIFO_SIZE_RX:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Words");

			if ((set) && ((set % 1024) == 0))
			{
				strcat(buf, " (");
				gsc_label_long_comma_buf(set / 1024, buf + strlen(buf));
				strcat(buf, "K)");
			}

			break;

		case HPDI32COS_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_HPDI32COS)
			{
				strcpy(buf, "HPDI32-COS");
			}
			else
			{
				errs	= 1;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case HPDI32COS_QUERY_FORM_FACTOR:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HPDI32COS_FORM_FACTOR_UNKNOWN:	ptr	= "Not Known";	break;
				case HPDI32COS_FORM_FACTOR_PCI:		ptr	= "PCI";		break;
				case HPDI32COS_FORM_FACTOR_PMC:		ptr	= "PMC";		break;
			}

			break;

		case HPDI32COS_QUERY_MODEL:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case HPDI32COS_MODEL_HPDI32A:	ptr	= "HPDI32A";		break;
				case HPDI32COS_MODEL_HPDI32B:	ptr	= "HPDI32B";		break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", ptr);
		else
			printf("%s\n", ptr);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


