// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/reg.c $
// $Rev: 54392 $
// $Date: 2024-05-01 14:51:34 -0500 (Wed, 01 May 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



#if 0
// For code format check.
	if (verbose
	if (verbose
#endif



// macros *********************************************************************

#define	_GSC_REG(a)		"GSC " #a, HPDI32COS_GSC_##a, 0, 0
#define	WIDTH			24

#define	SHOW_FIELD(begin,end,eol,opts,desc)	\
	gsc_reg_field_show(	\
		width + 10,		\
		WIDTH,			\
		value,			\
		(begin),		\
		(end),			\
		(eol),			\
		(opts),			\
		(desc))



//*****************************************************************************
static int _frr_detail(int fd, int supported, u32 value, int width)
{
	int		errs	= 0;
	long	v;

	SHOW_FIELD(31, 24, 1, NULL,		"Reserved"		);

	SHOW_FIELD(23, 16, 0, NULL,		"Board ID"		);
	v	= (value >> 16) & 0xFF;

	if (v == 0x10)
	{
		printf("HPDI32B-COS\n");
	}
	else
	{
		errs++;
		printf("ERROR <---  (0x%lX)\n", (long) v);
	}

	SHOW_FIELD(15,  8, 1, NULL,		"Reserved"		);

	SHOW_FIELD( 7,  0, 0, NULL,		"Firmware Rev"	);
	v	= value & 0xFF;
	printf("0x%02lX\n", (long) v);

	return(errs);
}



//*****************************************************************************
static int _bcr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	disabled[]	= { "Disabled",	"Enabled"	};
	static const char*	idle[]		= { "Idle",		"Active"	};
	static const char*	mode[]		= { "COS",		"LA"		};
	static const char*	output[]	= { "Output",	"Input"		};

	SHOW_FIELD(31, 17, 1, NULL,		"Reserved"			);
	SHOW_FIELD(16, 16, 1, mode,		"Mode"				);
	SHOW_FIELD(15, 15, 1, disabled,	"Tx Byte 3 Enable"	);
	SHOW_FIELD(14, 14, 1, disabled,	"Tx Byte 2 Enable"	);
	SHOW_FIELD(13, 13, 1, disabled,	"Tx Byte 1 Enable"	);
	SHOW_FIELD(12, 12, 1, disabled,	"Tx Byte 0 Enable"	);
	SHOW_FIELD(11,  8, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 7,  7, 1, disabled,	"Loopback"			);
	SHOW_FIELD( 6,  6, 1, output,	"Tx D5 & D6"		);
	SHOW_FIELD( 5,  5, 1, disabled,	"Receiver"			);
	SHOW_FIELD( 4,  4, 1, disabled,	"Transmit"			);
	SHOW_FIELD( 3,  3, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 2,  2, 1, idle,		"Rx FIFO Reset"		);
	SHOW_FIELD( 1,  1, 1, NULL,		"Reserved"			);
	SHOW_FIELD( 0,  0, 1, idle,		"Board Reset"		);
	return(0);
}



//*****************************************************************************
static int _bsr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	no[]	= { "No",	"Yes"	};
	static const char*	off[]	= { "Off",	"On"	};
	static const char*	yes[]	= { "Yes",	"No"	};

	SHOW_FIELD(31, 27, 1, NULL,	"Reserved"			);
	SHOW_FIELD(26, 26, 1, no,	"COS Detected"		);
	SHOW_FIELD(25, 25, 1, no,	"Counter To 0"		);
	SHOW_FIELD(24, 24, 1, no,	"Rx Running"		);
	SHOW_FIELD(23, 23, 1, no,	"LA Triggered"		);
	SHOW_FIELD(22, 22, 1, no,	"Rx Stopped"		);
	SHOW_FIELD(21, 21, 1, NULL,	"Reserved"			);
	SHOW_FIELD(20, 20, 1, no,	"Rx FIFO Overflow"	);
	SHOW_FIELD(19, 19, 1, no,	"Rx FIFO Underflow"	);
	SHOW_FIELD(18, 18, 1, NULL,	"Reserved"			);
	SHOW_FIELD(17, 17, 1, off,	"Jumper 1"			);
	SHOW_FIELD(16, 16, 1, off,	"Jumper 0"			);
	SHOW_FIELD(15, 15, 1, yes,	"Rx FIFO Full"		);
	SHOW_FIELD(14, 14, 1, yes,	"Rx FIFO AF"		);
	SHOW_FIELD(13, 13, 1, yes,	"Rx FIFO AE"		);
	SHOW_FIELD(12, 12, 1, yes,	"Rx FIFO Empty"		);
	SHOW_FIELD(11,  7, 1, NULL,	"Reserved"			);
	SHOW_FIELD( 6,  0, 1, NULL,	"CMD6-CMD0"			);
	return(0);
}



//*****************************************************************************
static int _ecr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 1, NULL,	"Event Count"	);
	return(0);
}



//*****************************************************************************
static int _rar_detail(int fd, int supported, u32 value, int width)
{
	u32	v;

	SHOW_FIELD(31, 16, 0, NULL,	"Almost Full"	);
	v	= (value & 0xFFFF0000) >> 16;
	printf("%ld  (0x%lX)\n", (long) v, (long) v);

	SHOW_FIELD(15,  0, 0, NULL,	"Almost Empty"	);
	v	= value & 0xFFFF;
	printf("%ld  (0x%lX)\n", (long) v, (long) v);

	return(0);
}



//*****************************************************************************
static int _scdr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 1, NULL,	"Divider"	);
	return(0);
}



//*****************************************************************************
static int _rdfr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Rx FIFO Data"	);
	printf("0x%08lX\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _tdor_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Tx Data"	);
	printf("0x%08lX\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _rdir_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Rx Cable Data"	);
	printf("0x%08lX\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _latr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"LA Trigger"	);
	printf("0x%08lX\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _dmr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Data Mask"	);
	printf("0x%08lX\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _ecivr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Initial Count"	);
	printf("%ld\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _rwcr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Count"	);
	printf("%ld\n", (long) value);
	return(0);
}



//*****************************************************************************
static int _icr_irq_detail(int fd, int supported, u32 value, int width, const char** list)
{
	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"				);
	SHOW_FIELD(23, 23, 1, list,	"COS Event Detected"	);
	SHOW_FIELD(22, 22, 1, list,	"Event Count To 0"		);
	SHOW_FIELD(21, 21, 1, list,	"Rx Running"			);
	SHOW_FIELD(20, 20, 1, list,	"LA Triggered"			);
	SHOW_FIELD(19, 19, 1, list,	"Rx Stopped"			);
	SHOW_FIELD(18, 18, 1, NULL,	"Reserved"				);
	SHOW_FIELD(17, 17, 1, list,	"Rx FIFO Overflow"		);
	SHOW_FIELD(16, 16, 1, list,	"Rx FIFO Underflow"		);
	SHOW_FIELD(15, 15, 1, list,	"Rx FIFO Full"			);
	SHOW_FIELD(14, 14, 1, list,	"Rx FIFO AF"			);
	SHOW_FIELD(13, 13, 1, list,	"Rx FIFO Not AE"		);
	SHOW_FIELD(12, 12, 1, list,	"Rx FIFO Not Empty"		);
	SHOW_FIELD(11,  7, 1, NULL,	"Reserved"				);
	SHOW_FIELD( 6,  6, 1, list,	"Cable Command 6"		);
	SHOW_FIELD( 5,  5, 1, list,	"Cable Command 5"		);
	SHOW_FIELD( 4,  4, 1, list,	"Cable Command 4"		);
	SHOW_FIELD( 3,  3, 1, list,	"Cable Command 3"		);
	SHOW_FIELD( 2,  2, 1, list,	"Cable Command 2"		);
	SHOW_FIELD( 1,  1, 1, list,	"Cable Command 1"		);
	SHOW_FIELD( 0,  0, 1, list,	"Cable Command 0"		);
	return(0);
}



//*****************************************************************************
static int _icr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Disabled",	"Enabled"	};

	_icr_irq_detail(fd, supported, value, width, list);
	return(0);
}



//*****************************************************************************
static int _isr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Idle",	"Active"	};

	_icr_irq_detail(fd, supported, value, width, list);
	return(0);
}



//*****************************************************************************
static int _rfsr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"FIFO Size"	);
	printf("%ld Words", (long) value);

	if ((value) && ((value % 1024) == 0))
		printf("  (%ldK)", (long) (value / 1024));

	printf("\n");
	return(0);
}



//*****************************************************************************
static int _rfwcr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31,  0, 0, NULL,	"Fill Level"	);
	printf("%ld Words", (long) value);

	if ((value) && ((value % 1024) == 0))
		printf("  (%ldK)", (long) (value / 1024));

	printf("\n");
	return(0);
}



//*****************************************************************************
static int _ielr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Level Triggerred",	"Edge Triggerred"	};

	_icr_irq_detail(fd, supported, value, width, list);
	return(0);
}



//*****************************************************************************
static int _ihlr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	list[]	= { "Low",	"High"	};

	_icr_irq_detail(fd, supported, value, width, list);
	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(FRR),	0,	_frr_detail,	"Firmware Revision Register"	},
	{ _GSC_REG(BCR),	0,	_bcr_detail,	"Board Control Register"		},
	{ _GSC_REG(BSR),	0,	_bsr_detail,	"Board Status Register"			},
	{ _GSC_REG(ECR),	0,	_ecr_detail,	"Event Count Register"			},
	{ _GSC_REG(RAR),	0,	_rar_detail,	"Rx Almost Register"			},
	{ _GSC_REG(SCDR),	0,	_scdr_detail,	"Sample Clock Divider Register"	},
	{ _GSC_REG(RDFR),	0,	_rdfr_detail,	"Rx Data FIFO Register"			},
	{ _GSC_REG(TDOR),	0,	_tdor_detail,	"Tx Data Output Register"		},
	{ _GSC_REG(RDIR),	0,	_rdir_detail,	"Rx Data Input Register"		},
	{ _GSC_REG(LATR),	0,	_latr_detail,	"LA Trigger Register"			},
	{ _GSC_REG(DMR),	0,	_dmr_detail,	"Data Mask Register"			},
	{ _GSC_REG(ECIVR),	0,	_ecivr_detail,	"Event Count Init Val Register"	},
	{ _GSC_REG(RWCR),	0,	_rwcr_detail,	"Rx Word Count Register"		},
	{ _GSC_REG(ICR),	0,	_icr_detail,	"Interrupt Control Register"	},
	{ _GSC_REG(ISR),	0,	_isr_detail,	"Interrupt Status Register"		},
	{ _GSC_REG(RFSR),	0,	_rfsr_detail,	"Rx FIFO Size Register"			},
	{ _GSC_REG(RFWCR),	0,	_rfwcr_detail,	"Rx FIFO Word Count Register"	},
	{ _GSC_REG(IELR),	0,	_ielr_detail,	"Interrupt Edge/Level Register"	},
	{ _GSC_REG(IHLR),	0,	_ihlr_detail,	"Interrupt High/Low Register"	},
	{ NULL, 0, 0, 0, 0, NULL, NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	hpdi32cos_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* hpdi32cos_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	hpdi32cos_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* hpdi32cos_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	hpdi32cos_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* hpdi32cos_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	hpdi32cos_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* hpdi32cos_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	hpdi32cos_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int hpdi32cos_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, hpdi32cos_reg_read);
	return(errs);
}



