// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/reg_mod.c $
// $Rev: 54350 $
// $Date: 2024-04-30 14:45:35 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32cos_reg_mod
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32COS_IOCTL_REG_MOD service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		reg		This is the register to access.
*
*		set		This is the value to apply via the mask.
*
*		mask	These are the bits to modify.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32cos_reg_mod(int fd, int index, int verbose, u32 reg, u32 set, u32 mask)
{
	gsc_reg_t	arg;
	int			errs;
	const char*	name;
	int			ret;

	if (verbose)
		gsc_label_index("Register Mod", index);

	arg.reg		= reg;
	arg.value	= set;
	arg.mask	= mask;
	ret			= hpdi32cos_ioctl(fd, HPDI32COS_IOCTL_REG_MOD, &arg);
	errs		= ret ? 1 : 0;

	if (verbose)
	{
		name	= hpdi32cos_reg_get_name(reg);
		printf(	"%s  (val 0x%08lX, mask 0x%08lX, %s)\n",
				errs ? "FAIL <---" : "PASS",
				(unsigned long) set,
				(unsigned long) mask,
				name);
	}

	return(errs);
}


