// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/rx_fifo_status.c $
// $Rev: 54350 $
// $Date: 2024-04-30 14:45:35 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32cos_rx_fifo_status
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32COS_IOCTL_RX_FIFO_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32cos_rx_fifo_status(int fd, int index, int verbose, s32* get)
{
	char		buf[128]	= "";
	int			errs;
	const char*	ptr			= buf;
	s32			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Rx FIFO Status", index);

	ret		= hpdi32cos_ioctl(fd, HPDI32COS_IOCTL_RX_FIFO_STATUS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case HPDI32COS_FIFO_STATUS_EMPTY:

			ptr	= "Empty";
			break;

		case HPDI32COS_FIFO_STATUS_AE:

			ptr	= "Almost Empty";
			break;

		case HPDI32COS_FIFO_STATUS_MEDIUM:

			ptr	= "Medium";
			break;

		case HPDI32COS_FIFO_STATUS_AF:

			ptr	= "Almost Full";
			break;

		case HPDI32COS_FIFO_STATUS_FULL:

			ptr	= "Full";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


