// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/rx_fifo_under.c $
// $Rev: 54403 $
// $Date: 2024-05-02 09:38:33 -0500 (Thu, 02 May 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32cos_rx_fifo_under
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32COS_IOCTL_RX_FIFO_UNDER service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32cos_rx_fifo_under(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128]	= "";
	int			errs;
	const char*	ptr			= buf;
	s32			ret;

	if (verbose)
		gsc_label_index("Rx FIFO Underflow", index);

	ret		= hpdi32cos_ioctl(fd, HPDI32COS_IOCTL_RX_FIFO_UNDER, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case HPDI32COS_FIFO_ERROR_NO:

			ptr	= "No";
			break;

		case HPDI32COS_FIFO_ERROR_YES:

			ptr	= "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


