// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32-COS/HPDI32-COS-1.x_GSC_DN/utils/rx_io_mode.c $
// $Rev: 54350 $
// $Date: 2024-04-30 14:45:35 -0500 (Tue, 30 Apr 2024) $

// HPDI32-COS: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	hpdi32cos_rx_io_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the HPDI32COS_IOCTL_RX_IO_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely? 0 = no, !0 = yes
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int hpdi32cos_rx_io_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128]	= "";
	int			errs;
	const char*	ptr			= buf;
	s32			ret;

	if (verbose)
		gsc_label_index("Rx I/O Mode", index);

	ret		= hpdi32cos_ioctl(fd, HPDI32COS_IOCTL_RX_IO_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case GSC_IO_MODE_PIO:

			ptr	= "PIO";
			break;

		case GSC_IO_MODE_BMDMA:

			ptr	= "Block Mode DMA";
			break;

		case GSC_IO_MODE_DMDMA:

			ptr	= "Demand Mode DMA";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


